
#include "IRremote.h"

// IR variables
int receiver = 3;             // Signal Pin of IR receiver
IRrecv irrecv(receiver);      // create instance of 'irrecv'
decode_results results;       // create instance of 'decode_results'

// Joystick/game variables
int xPos = 507;
int yPos = 507;
byte joyXPin = A0;
byte joyYPin = A1;
byte joySwitch = 2;
volatile byte clickCounter = -1;
int minMoveHigh = 530;
int minMoveLow = 490;
int currentSpeed = 550;         // Default = an average speed
int speedIncrement = 25;        // Amount to increase/decrease speed with Y input
unsigned long current = 0;      // Holds current timestamp
int wait = 40;                  // ms to wait between messages [Note: lower wait = faster framerate]
volatile bool buttonPressed = false; // Gauge if the button is pressed

void setup()
{
  Serial.begin( 9600 );
  pinMode( joySwitch, INPUT_PULLUP );
  attachInterrupt( 0, jump, FALLING );
  current = millis();           // Set up the current time
  // Set up infrared receiver:
  irrecv.enableIRIn(); // Start the receiver
  
} // setup


void loop()
{
  int xMovement = analogRead( joyXPin );
  int yPos = analogRead( joyYPin );
  
  // Handle the Joystick X movement regardless of timing:
  if ( xMovement > minMoveHigh || xMovement < minMoveLow )
  {
    xPos = xMovement;
  }

  // Only handle the IR movement if input has been received:
  if ( irrecv.decode( &results ) ) // have we received an IR signal?
  {
    translateIR( results ); 
    irrecv.resume();            // receive the next value
  }

  // Either output a "Jump" message or a movement message this frame.
  if( buttonPressed )
  {
    delay( 50 );                // Put this in the main loop to stop
                                //    the message from being erased
    buttonPressed = false;      // After the delay, continue the loop
    Serial.println( "Jump" );
  }
  else if( millis() > current + wait )
  {
    currentSpeed = yPos > minMoveLow && yPos < minMoveHigh // If only moved a little...
      ? currentSpeed              //    ...just return the current speed
      : getSpeed( yPos );         // Only change yPos if joystick moved significantly
    //int distance = ;
    Serial.print( (String) xPos + "," + (String) yPos + ',' 
        + (String) currentSpeed + '\n' );
    current = millis();
    
  }
  
} // loop


int getSpeed( int yPos )
{
  // Negative values indicate Joystick moved up
  if( yPos < minMoveLow )       // Interpreted "UP"
  {
    // Protect from going over maximum value
    return currentSpeed + speedIncrement > 1023 ? 1023 : currentSpeed + speedIncrement;
  }
  else if( yPos > minMoveHigh ) // Interpreted "Down"
  {
    // Protect from going under 0
    return currentSpeed - speedIncrement < 0 ? 0 : currentSpeed - speedIncrement;
  }
  
} // getSpeed


void jump()
{
  buttonPressed = true;         // Indicate button was pressed.
  
} // jump


// When a button is pressed on the remote, handle the proper response
void translateIR( decode_results results ) // takes action based on IR code received
{
    switch( results.value )
    {
        case 0xFFA25D:  // POWER
          break;
        case 0xFFE21D: Serial.println("FUNC/STOP"); break;
        case 0xFF629D: Serial.println("VOL+"); break;
        case 0xFF22DD: Serial.println("FAST BACK");    break;
        case 0xFF02FD: Serial.println("PAUSE");    break;
        case 0xFFC23D: Serial.println("FAST FORWARD");   break;
        case 0xFFE01F: Serial.println("DOWN");    break;
        case 0xFFA857: Serial.println("VOL-");    break;
        case 0xFF906F: Serial.println("UP");    break;
        case 0xFF9867: Serial.println("EQ");    break;
        case 0xFFB04F: Serial.println("ST/REPT");    break;
        case 0xFF6897:  // 0
            //Serial.println("0");
            break;
        case 0xFF30CF:
            //Serial.println("1");
            break;
        case 0xFF18E7:
            //Serial.println("2");
            currentSpeed += speedIncrement * 2;
            break;
        case 0xFF7A85:
            //Serial.println("3");
            break;
        case 0xFF10EF:
            //Serial.println("4");
            xPos = -900;
            break;
        case 0xFF38C7:
            //Serial.println("5");
            jump();
            break;
        case 0xFF5AA5:
            //Serial.println("6");
            xPos = 900;
            break;
        case 0xFF42BD:
            //Serial.println("7");
            break;
        case 0xFF4AB5: //Serial.println("8");
            currentSpeed -= speedIncrement * 2;
            break;
        case 0xFF52AD: //Serial.println("9");
        break;
        case 0xFFFFFFFF: //Serial.println(" REPEAT");
        break;  
      
        default: 
          //Serial.println(" other button   ");
          break;
  
    }// End switch

} //END translateIR


